import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import logging
from multiprocessing import Process
import signal
import threading
import time

from src.holon.HolonicAgent import HolonicAgent

class TestAgent(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)

def proc_run():
    def signal_handler(signal, frame):
        print("signal_handler 1")
        _terminate_lock.set()
        exit(0)
    signal.signal(signal.SIGINT, signal_handler)

    _terminate_lock = threading.Event()
    while not _terminate_lock.is_set():
        _terminate_lock.wait(1)
    # _terminate_lock.wait()
    # while not _terminate_lock.is_set():
    #     time.sleep(0.1)
    # while True:
    #     time.sleep(0.1)

if __name__ == '__main__':
    # Helper.init_logging()
    # logging.info('***** Main start *****')
    print('***** Test start *****')

    # def signal_handler(signal, frame):
    #     a.terminate()
    #     print('***** TestAgent stop *****')
    #     exit(0)
    # signal.signal(signal.SIGINT, signal_handler)
    def signal_handler(signal, frame):
        print("signal_handler")
        # exit(0)
    signal.signal(signal.SIGINT, signal_handler)

    a = TestAgent()
    a.start()
    # p = Process(target=proc_run)
    # p.start()
    # p1 = Process(target=proc_run)
    # p1.start()

    print('***** Test STOP *****')

    # while True:
    #     time.sleep(1)



from datetime import datetime
import logging
from logging.handlers import TimedRotatingFileHandler
import os
from pathlib import Path


logger = logging.getLogger('ABDI')


def ensure_directory(dir_path):
    if not os.path.exists(dir_path):
        os.mkdir(dir_path)
        logger.info(f"Directory '{dir_path}' created successfully.")


def init_logging(log_dir, log_level=logging.DEBUG):
    formatter = logging.Formatter(
        '%(levelname)1.1s %(asctime)s %(module)15s:%(lineno)03d %(funcName)15s) %(message)s',
        datefmt='%H:%M:%S')
    
    Path(log_dir).mkdir(parents=True, exist_ok=True)
    log_path = os.path.join(log_dir, "abdi.log")
    file_handler = TimedRotatingFileHandler(log_path, when="d")
    file_handler.setLevel(log_level)
    file_handler.setFormatter(formatter)

    console_handler = logging.StreamHandler()
    console_handler.setLevel(log_level)
    console_handler.setFormatter(formatter)
    
    global logger
    logger.addHandler(console_handler)
    logger.addHandler(file_handler)    
    logger.setLevel(log_level)

    return logger



import os, sys
# sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))
# sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

import logging

from src.holon import logger
from src.holon.HolonicAgent import HolonicAgent
from hearing.microphone import Microphone

class Hearing(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        self.head_agents.append(Microphone(cfg))
        # self.body_agents.append(VoiceToText(cfg))


    def _on_connect(self, client, userdata, flags, rc):
        client.subscribe("microphone.wave_path")

        super()._on_connect(client, userdata, flags, rc)


    def _on_topic(self, topic, data):
        if "microphone.wave_path" == topic:
            filepath = data
            logger.debug(f"wave_path:{filepath}")
            try:
                with open(filepath, "rb") as file:
                    file_content = file.read()
                self._publish("hearing.voice", file_content)
                os.remove(filepath)
            except Exception as ex:
                logger.exception(ex)

        super()._on_topic(topic, data)



import ast

from src.holon import logger
from src.holon.HolonicAgent import HolonicAgent
from brain import brain_helper


class Controller(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        self.active_subject = None
        self.registered_subjects = []


    def _on_connect(self, client, userdata, flags, rc):
        client.subscribe("dialog.knowledge")
        client.subscribe("brain.register_subject")
        client.subscribe("brain.unregister_subject")
        client.subscribe("brain.subject_done")
        super()._on_connect(client, userdata, flags, rc)


    def _on_topic(self, topic, data):
        if "dialog.knowledge" == topic:
            if self.active_subject:
                self._publish(f'{self.active_subject}.knowledge', data)
            else:
                knowledge = ast.literal_eval(data)
                if (subject := knowledge[0][0]) in self.registered_subjects:
                    self.active_subject = subject
                    logger.info(f"Active subject: {self.active_subject}")
                    self._publish(f'{self.active_subject}.knowledge', data)
                else:
                    logger.warning(f"Uknown subject: {subject}")
        elif "brain.register_subject" == topic:
            if not data in self.registered_subjects:
                self.registered_subjects.append(data)
                logger.info(f"Register subject: {data}")
        elif "brain.unregister_subject" == topic:
            if data in self.registered_subjects:
                self.registered_subjects.remove(data)
                logger.info(f"Unregister subject: {data}")
        elif "brain.subject_done" == topic:
            logger.info(f"subject: {self.active_subject} is done.")
            self.active_subject = None
            
        super()._on_topic(topic, data)



# import atexit
import inspect
from multiprocessing import Process
import os
import signal
import sys
import threading
import time 

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir) 

import logging

from abdi_config import AbdiConfig
from broker.notifier import BrokerNotifier
from broker.broker_maker import BrokerMaker
from core.Agent import Agent
from holon.Blackboard import Blackboard
from holon.HolonicDesire import HolonicDesire
from holon.HolonicIntention import HolonicIntention


logger = logging.getLogger("ABDI")

# def callback_with_error():
#     print("This callback will throw an error.")
#     # raise ValueError("Oops! An error occurred.")

# atexit.register(callback_with_error)


class HolonicAgent(Agent, BrokerNotifier) :
    def __init__(self, config:AbdiConfig=None, b:Blackboard=None, d:HolonicDesire=None, i: HolonicIntention=None):
        b = b or Blackboard()
        d = d or HolonicDesire()
        i = i or HolonicIntention()
        super().__init__(b, d, i)
        
        self.config = config if config else AbdiConfig(options={})
        self.head_agents = []
        self.body_agents = []
        self.run_interval_seconds = 1
        
        self.name = f'<{self.__class__.__name__}>'        
        self._agent_proc = None        
        self._broker = None


    def start(self, head=False):
        self._agent_proc = Process(target=self._run, args=(self.config,))
        self._agent_proc.start()
        
        # procs = [self._agent_proc]

        # for a in self.head_agents:
        #     procs.append(a.start())
        # for a in self.body_agents:
        #     procs.append(a.start())
        for a in self.head_agents:
            a.start()
        for a in self.body_agents:
            a.start()
        
        if head:
            try:
                self._agent_proc.join()
            except:
                logger.warning(f"{self.name} terminated.")
        # if head:
        #     try:
        #         self._agent_proc.join()
        #         # for proc in procs:
        #         #     proc.join()
        #     except:
        #         logger.warning(f"System terminated.")
        #     return None
        # else:
        #     return self._agent_proc


    # def start(self):
    #     logger.debug(f"...")
        
    #     self._agent_proc = Process(target=self._run, args=(self.config,))
    #     self._agent_proc.start()

    #     for a in self.head_agents:
    #         a.start()
    #     for a in self.body_agents:
    #         a.start()
            
    #     return self._agent_proc



# =====================
#  Instance of Process 
# =====================


    def _is_running(self):
        return not self._terminate_lock.is_set()


    def _run(self, config:AbdiConfig):
        self.config = config
        self._run_begin()
        self._running()
        self._run_end()
    

    def _run_begin(self):
        logger.debug(f"start")

        def signal_handler(signal, frame):
            logger.warning(f"{self.name} Ctrl-C: {self.__class__.__name__}")
            self._terminate()
        signal.signal(signal.SIGINT, signal_handler)

        self._terminate_lock = threading.Event()
        
        logger.debug(f"create broker")
        if broker_type := self.config.get_broker_type():
            self._broker = BrokerMaker().create_broker(
                broker_type=broker_type, 
                notifier=self)
            self._broker.start(options=self.config.options)
        
        logger.debug(f"start interval_loop")
        def interval_loop():
            while not self._terminate_lock.is_set():
                self._run_interval()
                time.sleep(self.run_interval_seconds)
        threading.Thread(target=interval_loop).start()
            
        logger.debug(f"done.")


    def _run_interval(self):
        pass


    def _running(self):
        logger.debug(f"Running ...")

    def _run_end(self):
        logger.debug(f"Run end ...")

        while not self._terminate_lock.is_set():
            self._terminate_lock.wait(1)

        self._broker.stop() 


    def _publish(self, topic, payload=None):
        return self._broker.publish(topic, payload)


    def _subscribe(self, topic, data_type="str"):
        return self._broker.subscribe(topic, data_type)
        

    def _terminate(self):
        logger.warn(f"{self.name}.")

        for a in self.head_agents:
            name = a.__class__.__name__
            self._publish(topic='terminate', payload=name)

        for a in self.body_agents:
            name = a.__class__.__name__
            self._publish(topic='terminate', payload=name)

        self._terminate_lock.set()



# ==================================
#  Implementation of BrokerNotifier 
# ==================================

    
    def _on_connect(self):
        logger.info(f"{self.name} Broker is connected.")
        
        self._subscribe("echo")
        self._subscribe("terminate")


    def _on_message(self, topic:str, payload):
        pass


    # def _on_topic(self, topic, data):
    #     if "terminate" == topic:
    #         if data:
    #             if type(self).__name__ == data:
    #                 self._terminate()
    #         else:
    #             self._terminate()



# ==================================
#  Others operation 
# ==================================


    def _convert_to_text(self, payload) -> str:
        if payload:
            data = payload.decode('utf-8', 'ignore')
        else:
            data = None
        
        return data
        


from src.holon.HolonicAgent import HolonicAgent
from brain.navi import Navigator
from brain.greeting import Greeting
from brain.controller import Controller


class Brain(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        self.head_agents.append(Controller(cfg))
        self.body_agents.append(Navigator(cfg))
        self.body_agents.append(Greeting(cfg))


    def _on_connect(self, client, userdata, flags, rc):
        # client.subscribe("dialog.knowledge")
        super()._on_connect(client, userdata, flags, rc)


    def _on_topic(self, topic, data):
        super()._on_topic(topic, data)



from holon.HolonicAgent import HolonicAgent
from tests.guide.voice.speaker import Speaker
from voice.ToneProcessing import ToneProcessing

class Voice(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        self.head_agents.append(Speaker(cfg))
        self.body_agents.append(ToneProcessing(cfg))



#Note: you need to be using OpenAI Python v0.27.0 for the code below to work
import ast
import re

import openai

openai.api_key = 'xxx'

def test_chat():
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        messages=[
                {"role": "system", "content": "You are a helpful assistant."},
                {"role": "user", "content": "Who won the world series in 2020?"},
                {"role": "assistant", "content": "The Los Angeles Dodgers won the World Series in 2020."},
                {"role": "user", "content": "Where was it played?"}
            ]
    )
    print(f"completion: {completion['choices'][0]['message']['content']}")

def test_chat1():
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        messages=[
                {"role": "system", "content": "You are a language teacher."},
                {"role": "assistant", "content": "Please translate the Chinese to English:"},
                {"role": "assistant", "content": "1.你有權保持緘默，毋需違背意志做任何陳述。"},
                {"role": "user", "content": "Answer: First, you have the right to remain silent and do not have to make any statement against your will."},
                # {"role": "user", "content": "Answer: Frist, dog you have the right to remain silent and doo not have to make any statement against your will."},
                {"role": "assistant", "content": "Are there grammatical errors in the answer? please list if any."},
                # {"role": "assistant", "content": "How well does this user translate? Just list your comment."},
                # {"role": "assistant", "content": "Are there spelling mistakes in the answer? please list the error words if any."},
            ]
    )
    print(f"completion: {completion['choices'][0]['message']}")
    # print(f"completion: {completion['choices'][0]['message']['content']}")


def test_chat2():
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        temperature=0,
        messages=[
                {"role": "system", "content": "You are a sentence analyzer."},
                {"role": "assistant", "content": "Convert user's sentence to (subject, predict, object) format following the rules below:"},
                {"role": "assistant", "content": "1. Subject, predict and object use only one word. 2. Predict uses infinitive verb, if it is a verb. 3. Respond ONLY in the requested format: (subject, predict, object), without any other wrods."},
                {"role": "user", "content": "I want to go to the zoo."},
            ]
    )
    # print(f"completion: {completion['choices'][0]['message']}")
    # print(f"completion: {completion['choices'][0]['message']['content']}")
    print(f"completion: {completion}")


def test_chat3():
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        temperature=0,
        messages=[
                {"role": "system", "content": "You are a sentence analyzer."},
                {"role": "assistant", "content": "Convert user's sentence to (subject, predict, object) format following the rules below:"},
                {"role": "assistant", "content": "1. Subject, predict and object use only one word."},
                {"role": "assistant", "content": "2. Predict uses infinitive verb, if it is a verb."},
                {"role": "assistant", "content": "3. Respond ONLY in the requested format: (subject, predict, object), without any other wrods."},
                {"role": "user", "content": "terminate system"},
            ]
    )
    # print(f"completion: {completion['choices'][0]['message']}")
    # print(f"completion: {completion['choices'][0]['message']['content']}")
    print(f"completion: {completion}")


def is_sentence(sentence):
    # completion = complete_sentence(f"Does this sentence have a predicate and object? '{sentence}'. Just answer yes or no.")
    # return "yes" in completion['choices'][0]['text'].lower()
    return yes_or_no(f"Does this sentence have a predicate and object?'{sentence}'")


def is_verb(sentence):
    # completion = complete_sentence(f"Is '{sentence}' just a verb?. Just answer yes or no.")
    # return "yes" in completion['choices'][0]['text'].lower()
    verbs = ['back']
    return True if sentence in verbs \
        else yes_or_no(f"Is '{sentence}' just a verb?")


def is_noun(sentence):
    # completion = complete_sentence(f"Is '{sentence}' just a noun?. Just answer yes or no.")
    # return "yes" in completion['choices'][0]['text'].lower()
    nouns = ['IRI']
    return True if sentence in nouns \
        else yes_or_no(f"Is '{sentence}' just a noun?")


def is_negative(sentence):
    return yes_or_no(f"Does '{sentence}' mean negative emotion?")


def is_positive(sentence):
    return yes_or_no(f"Does '{sentence}' mean positive emotion?")


def complete_sentence(sentence):
    print(f"complete_sentence: {sentence}")
    completion = openai.Completion.create(
        model = "text-davinci-003",
        prompt = sentence,
        max_tokens = 3,
        temperature = 0,
        top_p = 1,
        n = 1,
        # stream = False,
        # logprobs = None,
        # stop = "\n"
    )
    # return example:
    # {
    #   "choices": [
    #     {
    #       "finish_reason": "length",
    #       "index": 0,
    #       "logprobs": null,
    #       "text": "\n\nYes"
    #     }
    #   ],
    #   "created": 1684747093,
    #   "id": "cmpl-7Ivy9uUdvQHxcbClY6R5mu2W06J7s",
    #   "model": "text-davinci-003",
    #   "object": "text_completion",
    #   "usage": {
    #     "completion_tokens": 3,
    #     "prompt_tokens": 7,
    #     "total_tokens": 10
    #   }
    # }
    # print(f'{completion}')
    return completion


def yes_or_no(user_prompt):
    print(f"yes_or_no: {user_prompt}")

    user_prompt += ". Is yes or no?"
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        temperature=0,
        messages=[
                {"role": "system", "content": "You are a sentence analyzer."},
                {"role": "assistant", "content": "Just answer yes or no."},
                {"role": "assistant", "name": "example_user", "content": "Is 'park' a noun?"},
                {"role": "assistant", "name": "example_assistant", "content": "yes"},
                {"role": "assistant", "name": "example_user", "content": "Is 'shoot' a verb?"},
                {"role": "assistant", "name": "example_assistant", "content": "yes"},
                {"role": "assistant", "name": "example_user", "content": "Is 'I shot an arrow.' a sentence?"},
                {"role": "assistant", "name": "example_assistant", "content": "yes"},
                {"role": "assistant", "name": "example_user", "content": "Is 'eat' a noun?"},
                {"role": "assistant", "name": "example_assistant", "content": "no"},
                {"role": "user", "content": user_prompt},
            ]
    )
    # print(f"completion: {completion['choices'][0]['message']}")
    # print(f"completion: {completion['choices'][0]['message']['content']}")

    # print(f'{completion}')
    content = completion['choices'][0]['message']['content']
    result = 'yes' in content.lower()
    print(f"result: {'yes' if result else 'no'}")
    return result


def parse_to_triplet(user_prompt):
    print(f"parse_to_triplet: {user_prompt}")
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        temperature=0,
        messages=[
                {"role": "system", "content": "You are a sentence analyzer."},
                {"role": "assistant", "content": "Convert user's sentence to (subject, predict, object) format following the rules below:"},
                {"role": "assistant", "content": "1. Subject, predict and object use only one word."},
                {"role": "assistant", "content": "2. Predict uses infinitive verb, if it is a verb."},
                {"role": "assistant", "content": "3. Respond ONLY in the requested format: (subject, predict, object), without any other wrods."},
                {"role": "system", "name": "example_user", "content": "I want to go to the park."},
                {"role": "system", "name": "example_assistant", "content": "(I, go, park)"},
                {"role": "system", "name": "example_user", "content": "I'm going to the bathroom."},
                {"role": "system", "name": "example_assistant", "content": "(I, go, bathroom)"},
                {"role": "system", "name": "example_user", "content": "我晚餐想吃麥當勞漢堡。"},
                {"role": "system", "name": "example_assistant", "content": "(I, eat, McDonald's)"},
                {"role": "system", "name": "example_user", "content": "terminate system."},
                {"role": "system", "name": "example_assistant", "content": "(, terminate, system)"},
                {"role": "user", "content": f"Analyze: '{user_prompt}'"},
            ]
    )
    # print(f"completion: {completion}")
    # print(f"completion: {completion['choices'][0]['message']}")
    # print(f"completion: {completion['choices'][0]['message']['content']}")

    triplet = completion['choices'][0]['message']['content']
    result = triplet if '(' in triplet and ')' in triplet else None
    print(f"result: {result}, triplet: {triplet}")
    return result
    

def edit_sentence(sentence):
    print(f"edit_sentence: {sentence}")
    completion = openai.Edit.create(
        model = "text-davinci-edit-001",
        input = sentence,
        temperature = 0,
        instruction = "Fix to a correct sentence with subject, predicate and object."
    )
    # return example:
    # {
    #     "choices": [
    #         {
    #         "index": 0,
    #         "text": "back to the future\n"
    #         }
    #     ],
    #     "created": 1684753916,
    #     "object": "edit",
    #     "usage": {
    #         "completion_tokens": 16,
    #         "prompt_tokens": 25,
    #         "total_tokens": 41
    #     }
    # }
    # print(f'{completion}')
    editted = completion['choices'][0]['text'].strip()
    print(f"editted: {editted}")
    return editted


def is_keyword(prompt:str):
    print(f"is_keyword: {prompt}")
    keywords = [
        'yes',
        'no',
        # 'forward',
        # 'back',
        # 'turn',
        ]
    result = prompt.lower() in keywords
    print(f"is_keyword: {'yes' if result else 'no'}")
    return result


def parse_sentence(user_prompt:str):
    print(f"parse_sentence: {user_prompt}")

    if not user_prompt:
        return None
    
    if is_sentence(user_prompt):
        return parse_to_triplet(user_prompt)
    elif is_verb(user_prompt):
        return f"(, {user_prompt},)"
    elif is_noun(user_prompt):
        return f"(, , {user_prompt})"
    # elif is_keyword(user_prompt):
    #     return user_prompt.lower()
    else:
        return None
    

def understand(prompt):
    if is_keyword(prompt):
        triplet = f'(, {prompt},)'
    else:
        triplet = parse_sentence(prompt)
        if not triplet:        
            prompt2 = f"It {prompt}"
            triplet = parse_sentence(prompt2)
            if not triplet:        
                edited_prompt2 = edit_sentence(prompt2)
                triplet = parse_sentence(edited_prompt2)
                if not triplet:
                    if is_positive(prompt):
                        triplet = f'(, yes,)'
                    elif is_negative(prompt):
                        triplet = f'(, no,)'
                    else:
                        triplet = f'(, {prompt},)'

    print(f'triplet: {triplet}')
    tuple_0 = tuple(triplet[1:-1].split(','))
    tuple_1 = tuple(x.strip() if x else None for x in tuple_0)
    return tuple_1


if __name__ == 'x__main__':
    edit_sentence("back")


if __name__ == '__main__':
    print(f'***** {__file__} Start *****\n')

    # completion = test_chat4("I'm going to the convenience store")
    # print(f"completion: {completion['choices'][0]['message']['content']}")

    # completion = test_chat4("Open the door.")
    # print(f"completion: {completion['choices'][0]['message']['content']}")

    prompt = "I want to eat McDonald's for dinner"
    prompt = "我要吃麥當勞。"
    prompt = "Building"
    prompt = "I shoot a arrow"
    prompt = "stop the car"
    prompt = "悠悅光，擁有時尚純白的建築外觀"
    prompt = "Run"
    prompt = "I woulk like to eat Chinese food."
    prompt = "系統關機"
    prompt = "系統"
    prompt = "How many chat completion choices to generate for each input message?"
    prompt = "Yes"
    prompt = "Dog"
    prompt = "I love my Ting"
    prompt = "I want to go to eastern building"
    prompt = "you go back!"
    prompt = "go back from me"
    prompt = "I go back from you."
    prompt = "turn left"
    prompt = "back"
    prompt = "stop"
    prompt = "wonderful"
    prompt = "go ahead"
    prompt = "shutup"
    prompt = "我想吃大便"
    prompt = "estern"
    prompt = "Please take me to the park"
    prompt = "我要去公園"
    prompt = "我要吃西餐"
    prompt = "yes"
    prompt = "我要去樓上"
    triplet = understand(prompt)

    print(f"result: {triplet}")

    print(f'\n***** {__file__} Stop *****')



import ast
import threading

from src.holon import logger
from src.holon.HolonicAgent import HolonicAgent
from brain import brain_helper


class Greeting(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)


    def _on_connect(self, client, userdata, flags, rc):
        client.subscribe("greeting.knowledge")

        # self.publish('brain.register_subject', 'greeting')
        threading.Timer(1, lambda: self._publish('brain.register_subject', 'greeting')).start()
        super()._on_connect(client, userdata, flags, rc)


    def _on_topic(self, topic, data):
        if "greeting.knowledge" == topic:
            knowledge = ast.literal_eval(data)
            if knowledge[0][1] == 'normal':
                brain_helper.speak(self, f'Hello, have a good day.')
            elif knowledge[0][1] == 'happy':
                brain_helper.speak(self, f'Wonderful, very nice to meet you.')
            else:
                logger.info(f'Uknown greeting mood.')
            self._publish('brain.subject_done')

        super()._on_topic(topic, data)


    def _terminate(self):
        self._publish('brain.unregister_subject', 'greeting')
        super()._terminate()


import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

# import logging
import signal
import time

from guide import GuideMain
# import Helper

if __name__ == '__main__':
    # Helper.init_logging()
    # logging.info('***** Main start *****')
    print('***** Main start *****')

    def signal_handler(signal, frame):
        print("signal_handler")
    signal.signal(signal.SIGINT, signal_handler)

    a = GuideMain()
    a.start()

    # time.sleep(5)
    # a.terminate()



# import os, sys
# sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))

from src.holon.HolonicAgent import HolonicAgent
from dialog.nlu import Nlu


class DialogSystem(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        # self.head_agents.append(AudioOutput())
        # self.body_agents.append(AudioInput(cfg))
        self.body_agents.append(Nlu(cfg))



from TTS.api import TTS

# Running a multi-speaker and multi-lingual model

# List available 🐸TTS models and choose the first one
models = TTS.list_models()
[print(f"{i} {name}") for i, name in enumerate(models)]
# model_name = models[30] # tts_models/zh-CN/baker/tacotron2-DDC-GST
# model_name = models[24] # tts_models/es/mai/tacotron2-DDC
model_name = models[7] # tts_models/en/ek1/tacotron2
print(f"model_name: {model_name}")
# Init TTS
tts = TTS(model_name=model_name, gpu=True)
print(f"tts.languages: {tts.languages}")
print(f"tts.speakers: {tts.speakers}")

# Run TTS

# ❗ Since this model is multi-speaker and multi-lingual, we must set the target speaker and the language
# Text to speech with a numpy output
# wav = tts.tts("This is a test! This is also a test!!", speaker=tts.speakers[0], language=tts.languages[0])
# Text to speech to a file
# tts.tts_to_file(text="Hello world!", speaker=tts.speakers[0], language=tts.languages[0], file_path="output.wav")
text = "我們的戰士沒有流一滴血。"
text = "普京發誓懲罰「武裝叛亂」 瓦格納回擊「換總統」。"
text = "一觸即發的俄羅斯內戰突然出現轉機。在白俄羅斯總統盧卡申科斡旋下，瓦格納領導人普里戈任發表聲明，同意停止進軍莫斯科，下令部隊返回大本營。"
text = "普里戈任在聲明中說:「我們踏上正義遊行，24小時內，移動了200公里前往達莫斯科。 在那段時間裡，我們的戰士沒有流一滴血。 現在已經到了流血的時刻了。 不過，我們明白有責任避免俄羅斯人流血，正根據計劃掉頭返回我們的大本營。"
text = "Dónde puedo cambiar dinero?"
text = "Me gustaría cambiar 50 dólares a euros."
text = "The weather today is good."
tts.tts_to_file(text=text, file_path="output3-1.wav")



import logging
import time

from holon.HolonicAgent import HolonicAgent

class TestAgent(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)

    # def _run(self):
    #     logging.info(f"Run TestAgent")
    #     time.sleep(2)

if __name__ == '__main__':
    # Helper.init_logging()
    # logging.info('***** Main start *****')
    print('***** TestAgent start *****')

    a = TestAgent()
    a.start()



import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

import signal

from dialog.nlu import chatgpt
import guide_config


if __name__ == '__main__':
    # Helper.init_logging()
    # logging.info('***** Main start *****')
    print('***** RunTrans start *****')

    chatgpt.set_openai_api_key(guide_config.openai_api_key)

    def signal_handler(signal, frame):
        print("signal_handler")
    signal.signal(signal.SIGINT, signal_handler)

    prompt = "我要去公園"
    knowledge = chatgpt.understand(prompt)
    print(knowledge)



import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

import logging
from logging.handlers import TimedRotatingFileHandler
import signal

import Helper
from src.holon import AbdiConfig
from src.holon.HolonicAgent import HolonicAgent

from src.holon.HolonicAgent import HolonicAgent
# from visual.Visual import Visual
from hearing import Hearing
from voice import Voice
# from navi import Navigator
# from brain.navi import Navigator
from brain import Brain
from dialog import DialogSystem
import guide_config

class GuideMain(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        self.body_agents.append(DialogSystem(cfg))
        # self.head_agents.append(Visual())
        self.head_agents.append(Hearing(cfg))
        self.head_agents.append(Voice(cfg))
        # self.body_agents.append(Navigator(cfg))
        self.body_agents.append(Brain(cfg))


    def _on_connect(self, client, userdata, flags, rc):
        client.subscribe("guide.hearing.heared_text")
        client.subscribe("dialog.nlu.triplet")

        super()._on_connect(client, userdata, flags, rc)


    def _on_topic(self, topic, data):
        if "guide.hearing.heared_text" == topic:
            if '系統關機' in data:
                self._terminate()
        # elif "dialog.nlu.triplet" == topic:
        #     logging.info(f'### {data} ###')

        super()._on_topic(topic, data)


if __name__ == '__main__':
    logger = Helper.init_logging(log_dir=guide_config.log_dir, log_level=guide_config.log_level)
    logger.info(f'***** GuideMain start *****')

    def signal_handler(signal, frame):
        print("signal_handler")
    signal.signal(signal.SIGINT, signal_handler)

    cfg = AbdiConfig()
    cfg.mqtt_address = guide_config.mqtt_address
    cfg.mqtt_port = guide_config.mqtt_port
    cfg.mqtt_keepalive = guide_config.mqtt_keepalive
    cfg.mqtt_username = guide_config.mqtt_username
    cfg.mqtt_password = guide_config.mqtt_password
    os.environ["OPENAI_API_KEY"] = guide_config.openai_api_key

    a = GuideMain(cfg)
    a.start()



from multiprocessing import Process

from navi.walk.KanbanDetect import KanbanDetect
from navi.walk.RoadDetect import RoadDetect
from holon.HolonicAgent import HolonicAgent

class WalkGuide(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        self.body_agents.append(KanbanDetect(cfg))
        self.body_agents.append(RoadDetect(cfg))


    def start(self):
        self._agent_proc = Process(target=self._run, args=(self._config,))
        self._agent_proc.start()

        for a in self.head_agents:
            a.start()
        for a in self.body_agents:
            a.start()



#Note: you need to be using OpenAI Python v0.27.0 for the code below to work
import json
import os
import threading

import openai

from src.holon import logger

# import tests.lab.lab_config

def set_openai_api_key(key):
    print(f"set_openai_api_key: {key}")
    openai.api_key = key


def parse_to_triplet(user_prompt):
    print(f"parse_to_triplet: {user_prompt}")
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        temperature=0,
        messages=[
                {"role": "system", "content": "You are a sentence analyzer."},
                {"role": "assistant", "content": "Convert user's sentence to (subject, predict, object) format following the rules below:"},
                {"role": "assistant", "content": "1. Subject, predict and object use only one word."},
                {"role": "assistant", "content": "2. Predict uses infinitive verb, if it is a verb."},
                {"role": "assistant", "content": "3. Respond ONLY in the requested format: (subject, predict, object), without any other wrods."},
                {"role": "system", "name": "example_user", "content": "I want to go to the park."},
                {"role": "system", "name": "example_assistant", "content": "(I, go, park)"},
                {"role": "system", "name": "example_user", "content": "I'm going to the bathroom."},
                {"role": "system", "name": "example_assistant", "content": "(I, go, bathroom)"},
                {"role": "system", "name": "example_user", "content": "我晚餐想吃麥當勞漢堡。"},
                {"role": "system", "name": "example_assistant", "content": "(I, eat, McDonald's)"},
                {"role": "system", "name": "example_user", "content": "terminate system."},
                {"role": "system", "name": "example_assistant", "content": "(, terminate, system)"},
                {"role": "user", "content": f"Analyze: '{user_prompt}'"},
            ]
    )
    # print(f"completion: {completion}")
    # print(f"completion: {completion['choices'][0]['message']}")
    # print(f"completion: {completion['choices'][0]['message']['content']}")

    triplet = completion['choices'][0]['message']['content']
    result = triplet if '(' in triplet and ')' in triplet else None
    print(f"result: {result}, triplet: {triplet}")
    return result
    

def understand(prompt, last_sentence=None):
    global _subject
    global _predict
    global _object
    global _positivity
    global _classification

    print(f"openai.api_key: {openai.api_key}")
        

    def analyze_positivity(user_prompt):
        logger.info(f"analyze_positivity: {user_prompt}, last_sentence:{last_sentence}")

        if last_sentence:
            completion = openai.Completion.create(
                model="text-davinci-003",
                temperature=0,
                max_tokens=5,
                prompt=f"""Guide: '{last_sentence}'
User: '{user_prompt}'
Does that mean the user agrees or is positive? Just answer yes or no only."""
            )
        else:
            completion = openai.Completion.create(
                model="text-davinci-003",
                temperature=0,
                max_tokens=5,
                prompt=f"Is '{user_prompt}' a positive sentence or word? Just response yes or no."
                # prompt=f"Is the sentence express positive or negative?\n{user_prompt}"
                # prompt=f"Is the sentence express positive, negative or neutral?\n{user_prompt}"
            )

        # print(f'completion: {completion}')

        global _positivity
        text = completion['choices'][0]['text']
        #_positivity = 'pos' in text.lower()
        _positivity = 'yes' in text.lower()


    def classify_instruction(user_prompt):
        delimiter = "####"
        system_message = f"""
    You will receive an instruction from a user.
    The user's directive will be separated by {delimiter} characters.
    Please categorize the instruction into major and minor categories.
    And provide your output in json format with key values: primary (major category) and secondary (minor category).

    Primary (main category): go somewhere, get items, clean up the mess, provide information, greeting or unsupported categories.

    minor categories of greeting:
    normal
    happy

    minor categories of go somewhere:
    go to a park
    go to a entrance
    go to a toilet
    go to a export
    go to a restaurant

    minor categories of get items:
    take a book
    take a glass of water
    take the remote control
    take a fruit
    take some items

    minor categories of clean up the mess:
    clear the table
    clean up the ground
    clean windows
    clean others 

    minor categories of provide information:
    product specification
    price
    reviews
    restaurant suggestion
    others
    talk to real people

    """
        messages =  [  
            {'role':'system', 'content': system_message},    
            {'role':'user', 'content': f"{delimiter}{user_prompt}{delimiter}"},  
        ]

        completion = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            temperature=0,
            messages=messages
        )
        content = completion['choices'][0]['message']['content']
        global _classification
        _classification = tuple(json.loads(content).values())


    def _process_result(result):
        if result:
            result = result.strip()
            if result[0] == "(":
                result = result[1:]
            if result[-1] == ")":
                result = result[:-1]
        print(f"result: {result}")
        return result
        

    def parse_subject(user_prompt):
        print(f"parse_subject: {user_prompt}")
        completion = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            temperature=0,
            max_tokens=3,
            messages=[
                    {"role": "system", "content": "You are a sentence analyzer."},
                    {"role": "assistant", "content": "Convert user's sentence to (subject) format following the rules below:"},
                    {"role": "assistant", "content": "1. Response only one word."},
                    {"role": "assistant", "content": "2. If there is no subject, infer the subject."},
                    {"role": "assistant", "content": "3. Respond ONLY in the requested format: (subject), without any other wrods."},
                    {"role": "assistant", "content": "4. Answer in English"},
                    {"role": "system", "name": "example_user", "content": "I want to go to the park."},
                    {"role": "system", "name": "example_assistant", "content": "(I)"},
                    {"role": "system", "name": "example_user", "content": "He's going to the bathroom."},
                    {"role": "system", "name": "example_assistant", "content": "(He)"},
                    {"role": "system", "name": "example_user", "content": "我晚餐想吃麥當勞漢堡。"},
                    {"role": "system", "name": "example_assistant", "content": "(I)"},
                    {"role": "system", "name": "example_user", "content": "terminate system."},
                    {"role": "system", "name": "example_assistant", "content": "(You)"},
                    {"role": "user", "content": f"Analyze: \"{user_prompt}\", response only one word."},
                ]
        )

        global _subject
        _subject = _process_result(completion['choices'][0]['message']['content'])


    def parse_predict(user_prompt):
        print(f"parse_predict: {user_prompt}")
        completion = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            temperature=0,
            max_tokens=3,
            messages=[
                    {"role": "system", "content": "You are a sentence analyzer."},
                    {"role": "assistant", "content": "Convert user's sentence to (predict) format following the rules below:"},
                    {"role": "assistant", "content": "1. Response only one word."},
                    {"role": "assistant", "content": "2. If there is no predict, infer the subject."},
                    {"role": "assistant", "content": "3. Respond ONLY in the requested format: (predict), without any other wrods."},
                    {"role": "assistant", "content": "4. Answer in English"},
                    {"role": "system", "name": "example_user", "content": "I want to go to the park."},
                    {"role": "system", "name": "example_assistant", "content": "(go)"},
                    {"role": "system", "name": "example_user", "content": "He's going to the bathroom."},
                    {"role": "system", "name": "example_assistant", "content": "(go)"},
                    {"role": "system", "name": "example_user", "content": "我晚餐想吃麥當勞漢堡。"},
                    {"role": "system", "name": "example_assistant", "content": "(eat)"},
                    {"role": "system", "name": "example_user", "content": "terminate system."},
                    {"role": "system", "name": "example_assistant", "content": "(terminate)"},
                    {"role": "user", "content": f"Analyze: \"{user_prompt}\", response only one word."},
                ]
        )

        global _predict
        _predict = _process_result(completion['choices'][0]['message']['content'])
        

    def parse_object(user_prompt):
        print(f"parse_object: {user_prompt}")
        completion = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            temperature=0,
            max_tokens=4,
            messages=[
                    {"role": "system", "content": "You are a sentence analyzer."},
                    {"role": "assistant", "content": "Convert user's sentence to (object) format following the rules below:"},
                    {"role": "assistant", "content": "1. Response only one word."},
                    {"role": "assistant", "content": "2. If there is no object, infer the object."},
                    {"role": "assistant", "content": "3. Respond ONLY in the requested format: (object), without any other wrods."},
                    {"role": "assistant", "content": "4. Answer in English"},
                    {"role": "system", "name": "example_user", "content": "I want to go to the park."},
                    {"role": "system", "name": "example_assistant", "content": "(park)"},
                    {"role": "system", "name": "example_user", "content": "He's going to the bathroom."},
                    {"role": "system", "name": "example_assistant", "content": "(bathroom)"},
                    {"role": "system", "name": "example_user", "content": "我晚餐想吃麥當勞漢堡。"},
                    {"role": "system", "name": "example_assistant", "content": "(McDonald's hamburger)"},
                    {"role": "system", "name": "example_user", "content": "terminate system."},
                    {"role": "system", "name": "example_assistant", "content": "(system)"},
                    {"role": "user", "content": f"Analyze: \"{user_prompt}\", response only one word."},
                ]
        )

        global _object
        _object = _process_result(completion['choices'][0]['message']['content'])
    

    threads = []
    threads.append(threading.Thread(target=classify_instruction, args=(prompt,)))
    threads.append(threading.Thread(target=parse_subject, args=(prompt,)))
    threads.append(threading.Thread(target=parse_predict, args=(prompt,)))
    threads.append(threading.Thread(target=parse_object, args=(prompt,)))
    threads.append(threading.Thread(target=analyze_positivity, args=(prompt,)))

    for thread in threads:
        thread.start()
    for thread in threads:
        thread.join()

    # return (_subject, _predict, _object)
    return _classification, (_subject, _predict, _object, _positivity)


if __name__ == '__main__':
    print(f'***** {__file__} Start *****\n')

    set_openai_api_key(os.getenv('OPENAI_API_KEY'))

    prompt = "I want to eat McDonald's for dinner"
    prompt = "我要吃麥當勞。"
    prompt = "Building"
    prompt = "悠悅光，擁有時尚純白的建築外觀"
    prompt = "系統"
    prompt = "How many chat completion choices to generate for each input message?"
    prompt = "Yes"
    prompt = "I want to go to eastern building"
    prompt = "I go back from you."
    prompt = "back"
    prompt = "stop"
    prompt = "我想吃大便"
    prompt = "estern"
    prompt = "turn left"
    prompt = "Terminate the system"
    prompt = "Please take me to the park"
    prompt = "go back from me"
    prompt = "go ahead"
    prompt = "系統關機"
    prompt = "It's go back"
    prompt = "我要吃西餐"
    prompt = "stop the car"
    prompt = "I woulk like to eat Chinese food."
    prompt = "我要去樓上"
    prompt = "I shoot an arrow"
    prompt = "you go back!"
    prompt = "yes"
    prompt = "Run"
    prompt = "shutup"
    prompt = "I love my Ting"
    prompt = "wonderful"
    prompt = "Dog"
    prompt = "esto no está bien"
    prompt = "eso es bueno"
    prompt = "我要去公園"
    triplet = understand(prompt)

    print(f"result: {triplet}")

    print(f'\n***** {__file__} Stop *****')



from TTS.api import TTS

# Running a multi-speaker and multi-lingual model

# List available 🐸TTS models and choose the first one
model_name = TTS.list_models()[0]
print(f"model_name: {model_name}")
# Init TTS
tts = TTS(model_name=model_name, gpu=True)
print(f"tts.languages: {tts.languages}")

# Run TTS

# ❗ Since this model is multi-speaker and multi-lingual, we must set the target speaker and the language
# Text to speech with a numpy output
wav = tts.tts("This is a test! This is also a test!!", speaker=tts.speakers[0], language=tts.languages[0])
# Text to speech to a file
tts.tts_to_file(text="Hello world!", speaker=tts.speakers[0], language=tts.languages[0], file_path="output.wav")
# tts.tts_to_file(text="我們的戰士沒有流一滴血", speaker=tts.speakers[0], language=tts.languages[0], file_path="output.wav")

# Running a single speaker model

# Init TTS with the target model name
tts = TTS(model_name="tts_models/de/thorsten/tacotron2-DDC", progress_bar=False, gpu=True)
print(f"tts.languages: {tts.languages}")
# Run TTS
tts.tts_to_file(text="Ich bin eine Testnachricht.", file_path="output1.wav")

# Example voice cloning with YourTTS in English, French and Portuguese

tts = TTS(model_name="tts_models/multilingual/multi-dataset/your_tts", progr0ess_bar=False, gpu=True)
print(f"tts.languages: {tts.languages}")
# tts.tts_to_file("This is voice cloning.", speaker_wav="my/cloning/audio.wav", language="en", file_path="output2-1.wav")
# tts.tts_to_file("C'est le clonage de la voix.", speaker_wav="my/cloning/audio.wav", language="fr-fr", file_path="output2-2.wav")
# tts.tts_to_file("Isso é clonagem de voz.", speaker_wav="my/cloning/audio.wav", language="pt-br", file_path="output2-3.wav")


# Example voice conversion converting speaker of the `source_wav` to the speaker of the `target_wav`

# tts = TTS(model_name="voice_conversion_models/multilingual/vctk/freevc24", progress_bar=False, gpu=True)
# tts.voice_conversion_to_file(source_wav="my/source.wav", target_wav="my/target.wav", file_path="output.wav")

# Example voice cloning by a single speaker TTS model combining with the voice conversion model. This way, you can
# clone voices by using any model in 🐸TTS.

# tts = TTS("tts_models/de/thorsten/tacotron2-DDC")
# tts.tts_with_vc_to_file(
#     "Wie sage ich auf Italienisch, dass ich dich liebe?",
#     speaker_wav="target/speaker.wav",
#     file_path="ouptut.wav"
# )

# Example text to speech using [🐸Coqui Studio](https://coqui.ai) models.

# You can use all of your available speakers in the studio.
# [🐸Coqui Studio](https://coqui.ai) API token is required. You can get it from the [account page](https://coqui.ai/account).
# You should set the `COQUI_STUDIO_TOKEN` environment variable to use the API token.

# If you have a valid API token set you will see the studio speakers as separate models in the list.
# The name format is coqui_studio/en/<studio_speaker_name>/coqui_studio
# models = TTS().list_models()
# # Init TTS with the target studio speaker
# tts = TTS(model_name="coqui_studio/en/Torcull Diarmuid/coqui_studio", progress_bar=False, gpu=False)
# print(f"tts.languages: {tts.languages}")
# # Run TTS
# tts.tts_to_file(text="This is a test.", file_path="output3-1.wav")
# # Run TTS with emotion and speed control
# tts.tts_to_file(text="This is a test.", file_path="output3-2.wav", emotion="Happy", speed=1.5)


# #Example text to speech using **Fairseq models in ~1100 languages** 🤯.

# #For these models use the following name format: `tts_models/<lang-iso_code>/fairseq/vits`.
# #You can find the list of language ISO codes [here](https://dl.fbaipublicfiles.com/mms/tts/all-tts-languages.html) and learn about the Fairseq models [here](https://github.com/facebookresearch/fairseq/tree/main/examples/mms).

# # TTS with on the fly voice conversion
# api = TTS("tts_models/deu/fairseq/vits")
# api.tts_with_vc_to_file(
#     "Wie sage ich auf Italienisch, dass ich dich liebe?",
#     speaker_wav="target/speaker.wav",
#     file_path="ouptut.wav"
# )


import logging

def xinfo_decorator(original_func):
    def wrapper(self, *args, **kwargs):
        # 修改传入的消息
        # if len(args) > 0 and isinstance(args[0], str):
        args = (f"{self.__class__.__name__}: ",)
        # return original_func(self, *args, **kwargs)
        return original_func(self, *args)
    return wrapper

# def info_decorator(self, original_func):
#     def wrapper(msg0):
#         print(f'msg: {msg0}')
#         # if len(args) > 0:
#         #     print(f'args[0]: {args[0]}')
#         # if len(args) > 0 and isinstance(args[0], str):
#         #     args = ("AAA: " + args[0],) + args[1:]
#         # original_func(self, *args, **kwargs)
#         original_func(msg=self.__class__.__name__ + msg0)
#     return wrapper

class MyClass:
    def info_decorator(self, original_func):
        def wrapper(msg0):
            print(f'msg: {msg0}')
            original_func(msg=self.__class__.__name__ + msg0)
        return wrapper

    def __init__(self):
        # 创建自定义的日志记录器
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)

        # 创建格式化器
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

        # 创建处理器，并将格式化器关联到处理器
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(formatter)

        # 将处理器添加到日志记录器
        self.logger.addHandler(console_handler)

        # 对日志记录器的 info 方法应用装饰器
        self.logger.info = self.info_decorator(self.logger.info)

    def some_method(self):
        self.logger.info("message 1")
        self.logger.info("message 2")

my_object = MyClass()
my_object.some_method()



from abc import ABC, abstractmethod
from broker.notifier import BrokerNotifier


class MessageBroker(ABC):
    def __init__(self, notifier:BrokerNotifier):
        self._notifier = notifier
        
    
    @abstractmethod
    def start(self, options:dict):
        """Start the message broker."""
        
    
    @abstractmethod
    def stop(self):
        """Stop the message broker."""
        
    
    @abstractmethod
    def publish(self, topic:str, payload):
        """Publish the topic."""
        
    
    @abstractmethod
    def subscribe(self, topic:str, data_type):
        """Subscribe the topic."""



import logging

from broker import BrokerType


class AbdiConfig:
    def __init__(self, options=None):
        self.options = options if options else {}

        self.log_level = logging.DEBUG
        self.log_dir = "./_log"
        
        
    def get_broker_type(self) -> BrokerType:
        if broker_type := self.get("broker_type"):
            return BrokerType(broker_type.lower())
        else:
            return BrokerType.Empty
        
    
    def get(self, key:str):
        return self.options.get(key)
        
    
    def set(self, key:str, value):
        self.options[key] = value



import ast
import threading

import Helper
from Helper import logger
from src.holon.HolonicAgent import HolonicAgent
from navi.VisualInput import VisualInput
from navi.RouteFind import RouteFind
from navi.walk.WalkGuide import WalkGuide
from brain import brain_helper

class Navigator(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        self.state = 0
        # self.head_agents.append(VisualInput(cfg))
        # self.body_agents.append(WalkGuide(cfg))
        # self.body_agents.append(RouteFind(cfg))


    def _on_connect(self, client, userdata, flags, rc):
        # client.subscribe("dialog.nlu.triplet")
        client.subscribe("go somewhere.knowledge")        
        threading.Timer(2, lambda: self._publish('brain.register_subject', 'go somewhere')).start()

        super()._on_connect(client, userdata, flags, rc)


    def __is_go(self, predict):
        logger.debug(f"predict: {predict}")
        result = ("go" == predict or "take" == predict)
        return result


    def __set_state(self, new_state):
        self.state = new_state
        logger.debug(f"New state: {new_state}")
       
    
    def __process_navi(self, knowledge):
        logger.debug(f"state: {self.state}, knowledge: '{knowledge}'")
        triplet = knowledge[1]
        if self.state == 0:
            self.target = triplet[2]
            brain_helper.speak(self, f"How about going to Dragon {self.target}?")
            self.__set_state(1)
        elif self.state == 1:
            if triplet[3]:
                brain_helper.speak(self, f"OK, let's go.")
                def arrive():
                    self.__set_state(0)
                    brain_helper.speak(self, f"We arrive the Dragon {self.target}.")
                    self._publish('brain.subject_done')
                threading.Timer(6, lambda: arrive()).start()
                self.__set_state(2)
            else:
                brain_helper.speak(self, f"Let me know if you want to go to the {self.target}.")
                self.__set_state(0)
                self._publish('brain.subject_done')
        elif self.state == 2:
            brain_helper.speak(self, f"We are on our way to Dragon {self.target}.")


    def _on_topic(self, topic, data):
        if "go somewhere.knowledge" == topic:
            knowledge = ast.literal_eval(data)
            # if (self.state == 0 and knowledge[0][0] == 'go somewhere') or self.state != 0:
                # logger.info(f"process: {data}")
                # if knowledge[0]
            self.__process_navi(knowledge)

        super()._on_topic(topic, data)


if __name__ == '__main__':
    Helper.init_logging()
    logger.info('***** Hearing start *****')
    a = Navigator()
    ans = a.__is_go('go')
    print(f'Is go: {ans}')


    # def __is_go(self, predict):
    #     logging.debug(f"predict: {predict}")
    #     completion = openai.ChatCompletion.create(
    #         model="gpt-3.5-turbo",
    #         temperature=0,
    #         max_tokens=3,
    #         messages=[
    #                 {"role": "system", "content": "You are a word analyzer."},
    #                 {"role": "assistant", "content": "Check the word sentence to (subject) format following the rules below:"},
    #                 {"role": "assistant", "content": "1. Response only yes or no."},
    #                 {"role": "assistant", "content": "2. If there is no subject, infer the subject."},
    #                 {"role": "assistant", "content": "3. Respond ONLY in the requested format: (subject), without any other wrods."},
    #                 {"role": "assistant", "content": "4. Answer in English"},
    #                 {"role": "system", "name": "example_user", "content": "I want to go to the park."},
    #                 {"role": "system", "name": "example_assistant", "content": "(I)"},
    #                 {"role": "system", "name": "example_user", "content": "He's going to the bathroom."},
    #                 {"role": "system", "name": "example_assistant", "content": "(He)"},
    #                 {"role": "system", "name": "example_user", "content": "我晚餐想吃麥當勞漢堡。"},
    #                 {"role": "system", "name": "example_assistant", "content": "(I)"},
    #                 {"role": "system", "name": "example_user", "content": "terminate system."},
    #                 {"role": "system", "name": "example_assistant", "content": "(You)"},
    #                 {"role": "user", "content": f"Analyze: \"{predict}\", response only one word."},
    #             ]
    #     )

    #     resp = completion['choices'][0]['message']['content']
    #     logging.debug(f"resp: {resp}")
    #     return False



from abc import ABC, abstractmethod


class BrokerNotifier(ABC):
        
    def __init__(self):
        pass
        
    
    @abstractmethod
    def _on_connect(self):        
        """Called when connected to the broker."""
        
    
    @abstractmethod
    def _on_message(self, topic:str, payload):        
        """Called when a message comes in."""



import setuptools

with open("README.md", "r", encoding = "utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name = "agent-bdi",
    version = "2.0.3",
    author = "Ming Fang Shiu",
    author_email='avatar.xu@gmail.com',
    description='Agent BDI framework',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/mfshiu/abdi.git",
    project_urls = {
        "Bug Tracker": "https://bugtracker.zoho.com/portal/avatardotxugmaildotcom#allprojects/1982079000000043717/",
    },
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # package_dir = {"": "src"},
    # packages = setuptools.find_packages(where="src"),
    python_requires = ">=3.9"
)


import logging

mqtt_address = "localhost"
mqtt_port = 1883
mqtt_keepalive = 60
mqtt_username = "eric"
mqtt_password = "eric123"

log_level = logging.DEBUG
log_dir = "D:\\Work\\SDK\\agent-bdi\\tests\guide\\_log"

openai_api_key = 'openai_api_key'



import os
# import os, sys
# sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))

import logging

from src.holon import logger
from src.holon.HolonicAgent import HolonicAgent
import dialog.nlu.chatgpt as chatgpt
# import DialogSession

class Nlu(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        # self.head_agents.append(DialogSession())

        self.last_sentence = ""


    def _on_connect(self, client, userdata, flags, rc):
        client.subscribe("hearing.trans.text")
        client.subscribe("voice.text")

        chatgpt.set_openai_api_key(os.getenv('OPENAI_API_KEY'))
        
        super()._on_connect(client, userdata, flags, rc)


    def _on_topic(self, topic, data):
        if "hearing.trans.text" == topic:
            logger.debug(f"{self.name} heared '{data}'")
            knowledge = self._understand(data, self.last_sentence)
            # self.publish("dialog.nlu.triplet", str(knowledge[1]))
            self._publish("dialog.knowledge", str(knowledge))
            logger.info(f"Understand: {knowledge}")
        elif "voice.text" == topic:
            logger.info(f"System ask: {data}")
            self.last_sentence = data

        super()._on_topic(topic, data)


    def _understand(self, sentence, last_sentence):
        try:
            knowledge = chatgpt.understand(sentence, last_sentence)
            # logger.info(f'knowledge: {knowledge}')
        except Exception as ex:
            knowledge = None
            logger.exception(f"Error: {str(ex)}")
        return knowledge


# if __name__ == '__main__':
#     Helper.init_logging()
#     logger.info('***** Hearing start *****')
#     a = Hearing()
#     a.start()



import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))
from datetime import datetime as dt
import queue
from multiprocessing import Process
import time

import whisper
import torch

from src.holon.HolonicAgent import HolonicAgent
from src.holon import AbdiConfig, logger
import guide_config

device = "cuda" if torch.cuda.is_available() else "cpu"
# whisper.DecodingOptions(language="zh")
global whisper_model
whisper_model = whisper.load_model("small", device=device)
# whisper_model = whisper.load_model("medium", device=device)

class Transcriptionist(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)


    def _on_connect(self, client, userdata, flags, rc):
        client.subscribe("hearing.voice")
        client.subscribe("trans.test")

        super()._on_connect(client, userdata, flags, rc)


    def _on_message(self, client, db, msg):
        if "hearing.voice" == msg.topic:
            wave_path = dt.now().strftime("tests/_input/voice-%m%d-%H%M-%S.wav")
            # logging.debug(f'data: {data}')
            with open(wave_path, "wb") as file:
                file.write(msg.payload)
            self.wave_queue.put(wave_path)
        elif "trans.test" == msg.topic:
            self._publish("trans.test", 'publish ')


    def _run_begin(self):
        super()._run_begin()
        logger.info(f"device:{device}")
        self.wave_queue = queue.Queue()


    def _running(self):
        global whisper_model
        while self._is_running():
            if self.wave_queue.empty():
                time.sleep(.1)
                continue
            try:
                wave_path = self.wave_queue.get()
                logger.debug(f'transcribing wave_path: {wave_path}')
                result = whisper_model.transcribe(wave_path)
                logger.debug(f'result: {result}')
                # transcribed_text = str(result["text"].encode('utf-8'))[2:-1].strip()
                transcribed_text = result["text"]
                # logging.debug(f'running addr: {self._config.mqtt_address}')
                self._publish("hearing.trans.text", transcribed_text)        
                logger.info(f">>> \033[33m{transcribed_text}\033[0m")
                if os.path.exists(wave_path):
                    os.remove(wave_path)
                logger.debug(f'Remained waves: {self.wave_queue.qsize()}')
            except queue.Empty:
                pass
            except UnicodeEncodeError:
                logger.info(f">>> \033[33m{transcribed_text.encode('utf-8')}\033[0m")
            except Exception as ex:
                _, exc_value, _ = sys.exc_info()
                logger.error(exc_value)



import logging
import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
import signal

from src.holon import AbdiConfig
from tests.guide import Helper
import guide_config
from hearing.microphone import Microphone


if __name__ == '__main__':
    print('***** run_mic start *****')

    Helper.init_logging(log_dir='/home/ericxu/work/_log', log_level=logging.DEBUG)

    def signal_handler(signal, frame):
        print("signal_handler")
    signal.signal(signal.SIGINT, signal_handler)

    cfg = AbdiConfig()
    cfg.mqtt_address = guide_config.mqtt_address
    cfg.mqtt_port = guide_config.mqtt_port
    cfg.mqtt_keepalive = guide_config.mqtt_keepalive
    cfg.mqtt_username = guide_config.mqtt_username
    cfg.mqtt_password = guide_config.mqtt_password
    cfg.log_level = guide_config.log_level
    cfg.log_dir = guide_config.log_dir    
    os.environ["OPENAI_API_KEY"] = guide_config.openai_api_key

    a = Microphone(cfg)
    a.start()

    # time.sleep(5)
    # a.terminate()



import os
from datetime import datetime as dt

from TTS.api import TTS

from src.holon import logger
from src.holon.HolonicAgent import HolonicAgent
from voice.speaker import Speaker

class Voice(HolonicAgent):
    def __init__(self, cfg):
        super().__init__(cfg)
        self.head_agents.append(Speaker(cfg))
        # self.body_agents.append(VoiceToText(cfg))


    def _on_connect(self, client, userdata, flags, rc):
        client.subscribe("voice.text")

        self.models = TTS.list_models()
        self.tts = TTS(model_name=self.models[7], gpu=True)

        super()._on_connect(client, userdata, flags, rc)


    def _on_topic(self, topic, data):
        if "voice.text" == topic:
            filepath = dt.now().strftime("tests/_output/voice-%m%d-%H%M-%S.wav")
            logger.debug(f"voice_path:{filepath}")
            try:
                self.tts.tts_to_file(text=data, file_path=filepath)
                with open(filepath, "rb") as file:
                    file_content = file.read()
                self._publish("voice.wave", file_content)
                os.remove(filepath)
            except Exception as ex:
                logger.exception(ex)

        super()._on_topic(topic, data)



import json
import openai


def get_completion_messages(user_prompt):
    delimiter = "####"
    system_message = f"""
你將會收到一些客戶服務查詢。
每個客戶服務查詢將以 {delimiter} 字元進行分隔。
請將每個查詢分類為主要類別和次要類別。
並以 json 格式提供你的輸出，鍵值為：primary（主類別）和 secondary（次類別）。

Primary（主類別）: 帳單、技術支援、帳戶管理或一般查詢

帳單的次類別:
取消訂閱或升級
增加付款方式
收費說明
爭議收費

技術支援次類別:
一般故障排除
設備相容性
軟體更新

帳戶管理次要類別:
重設密碼
更新個人資訊
關閉帳戶
帳戶安全

一般查詢的次要類別:
產品資訊
價格
反饋
與真人客服對話

"""
    messages =  [  
        {'role':'system', 'content': system_message},    
        {'role':'user', 'content': f"{delimiter}{user_prompt}{delimiter}"},  
    ]
    return messages


def get_completion_messages1(user_prompt):
    delimiter = "####"
    system_message = f"""
你將會收到一些使用者的指令。
每個使用者的指令將以 {delimiter} 字元進行分隔。
請將每個查詢分類為主要類別和次要類別。
並以 json 格式提供你的輸出，鍵值為：primary（主類別）和 secondary（次類別）。

Primary（主類別）: 帶路導覽、取得物品、清理環境、提供資訊、打招呼或不屬於服務類別

打招呼的次類別:
一般
快樂

帶路導覽的次類別:
帶去公園
帶去入口
帶去廁所
帶去出口
帶去餐廳

取得物品次要類別:
拿一本書
拿一杯水
拿搖控器
拿一顆蘋果

清理環境次要類別:
清理桌子
清理地面
清理窗戶
清理牆壁

提供資訊的次要類別:
產品規格資訊
價格資訊
評論資訊
與真人對話
提供餐廳位置
提供娛樂場所位置

"""
    messages =  [  
        {'role':'system', 'content': system_message},    
        {'role':'user', 'content': f"{delimiter}{user_prompt}{delimiter}"},  
    ]
    return messages


def get_completion_messages2(user_prompt):
    delimiter = "####"
    system_message = f"""
You will receive an instruction from a user.
The user's directive will be separated by {delimiter} characters.
Please categorize the demand into major and minor categories.
And provide your output in json format with key values: primary (major category) and secondary (minor category).

Primary (main category): go somewhere, get items, clean up the mess, provide information, greeting or unsupported categories.

minor categories of greeting:
normal
happy

minor categories of lead the way:
go to a park
go to a entrance
go to a toilet
go to a export
go to a restaurant

minor categories of get items:
take a book
take a glass of water
take the remote control
take a fruit
take some items

minor categories of clean up the mess:
clear the table
clean up the ground
clean windows
clean others 

minor categories of provide information:
product specification
price
reviews
restaurant suggestion
others
talk to real people

"""
    messages =  [  
        {'role':'system', 'content': system_message},    
        {'role':'user', 'content': f"{delimiter}{user_prompt}{delimiter}"},  
    ]
    return messages


def get_triplet_completion_messages(user_prompt):
    delimiter = "####"
    system_message = f"""
你將會收到使用者所說的話。
使用者所說的話將以 {delimiter} 字元進行分隔。
請將使用者所說的話以 json 格式提供你的輸出，鍵值為：subject（主語）、predict（謂語）和 object（賓語）。
"""
    messages =  [  
        {'role':'system', 'content': system_message},    
        {"role": "assistant", "content": "注意以下規範："},
        {"role": "assistant", "content": "1. Use ONLY ONE English word for the value of each key."},
        {"role": "assistant", "content": "2. If there is no subject, infer the subject."},
        {"role": "assistant", "content": "3. Answer in English"},
        {"role": "system", "name": "example_user", "content": "I would like to go to a park."},
        {"role": "system", "name": "example_assistant", "content": """{"subject": "I",
"predict": "would like",
"object": "park"""},
        {"role": "system", "name": "example_user", "content": "He's going to the bathroom."},
        {"role": "system", "name": "example_assistant", "content": """{"subject": "he",
"predict": "go",
"object": "bathroom\""""},
        {'role':'user', 'content': f"{delimiter}{user_prompt}{delimiter}"},  
    ]
    return messages


if __name__ == '__main__':
    openai.api_key = ""

    user_prompt = "I would like to watch TV."    # O
    user_prompt = "I would like to go to a park."    # O
    user_prompt = "I would like to go to school."    # O
    user_prompt = "I have to see a doctor right now."    # O
    user_prompt = "I need to pee now."    # O
    user_prompt = "I want to read harry potter"    # O 
    user_prompt = "The floor is dirty, please clean it"  # O
    user_prompt = "please clean up these clutter"  # O
    user_prompt = "I want you to delete my profile and all user data" # X
    user_prompt = "So boring, I want to find a place to have fun."    # O
    user_prompt = "I'm hungry, I need to eat"
    user_prompt = "I'm hungry"    # O
    user_prompt = "hello, how are you?"    # O
    user_prompt = "hi good morning nice to meet you"    # O
    user_prompt = "wow good morning very nice to meet you"    # O
    user_prompt = "I was so glad to meet you"    # O
    user_prompt = "good morning"
    user_prompt = "i want diarrhea"   
    user_prompt = "my stomach hurts" 
    user_prompt = "I'm so thirsty"    # O
    user_prompt = "someone spilled a drink on the table"    # O
    user_prompt = "I want to watch TV"    # X
    user_prompt = "How much is the latest MacBook Pro?"    # O
    user_prompt = "i want to go for a walk with the dog."    # O

    messages = get_completion_messages2(user_prompt)
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        temperature=0,
        messages=messages
    )
    classfication = completion['choices'][0]['message']['content']
    values_tuple = tuple(json.loads(classfication).values())
    print(f"Classfication:\n{values_tuple}")


if __name__ == 'x__main__':
    openai.api_key = ""

    user_prompt = "我要你刪除我的個人資料和所有使用者資料" # X
    user_prompt = "地上很髒請打掃一下"  # O
    user_prompt = "我想要去遛狗"    # O
    user_prompt = "最新的MacBook Pro一台多少錢？"    # O
    user_prompt = "我要看電視"    # X
    user_prompt = "有人在桌上打翻了飲料"    # O
    user_prompt = "我想閱讀哈利波特"    # O 
    user_prompt = "我的口好渴"    # O
    user_prompt = "I would like to watch TV."    # O
    user_prompt = "我的肚子好痛"    # X
    user_prompt = "我想拉肚子"    # X
    user_prompt = "早安"    # O
    user_prompt = "嗨早安，見到你真開心"    # O
    user_prompt = "哈囉，你好"    # O
    user_prompt = "我肚子餓了"    # O
    user_prompt = "我要找一間餐廳吃飯"    # O
    user_prompt = "請帶我去一間餐廳"    # O
    user_prompt = "好無聊"    # O
    user_prompt = "好無聊，我想找個地方的玩"    # O
    user_prompt = "我要找地方尿尿"    # O
    user_prompt = "我要去公園"    # O
    user_prompt = "I would like to go to a park."    # O

    messages = get_completion_messages1(user_prompt)
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        temperature=0,
        messages=messages
    )
    classfication = completion['choices'][0]['message']['content']
    print(f"Classfication:\n{classfication}")

    messages = get_triplet_completion_messages(user_prompt)
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        temperature=0,
        messages=messages
    )
    classfication = completion['choices'][0]['message']['content']
    print(f"\nTriplet:\n{classfication}")
